@@ -136,6 +136,7 @@ module Test.QuickCheck
   , sample
   , sample'
 
+#ifndef NO_GADTS
     -- * The 'Function' typeclass: generation of random shrinkable, showable functions
 
     -- | Example of use:
@@ -145,7 +146,7 @@ module Test.QuickCheck
     -- >>>     prop (Fun _ f) = f "monkey" == f "banana" || f "banana" == f "elephant"
     -- >>> :}
     -- >>> quickCheck prop
-    -- *** Failed! Falsifiable (after 3 tests and 134 shrinks):
+    -- *** Failed! Falsified (after 3 tests and 134 shrinks):
     -- {"elephant"->1, "monkey"->1, _->0}
     --
     -- To generate random values of type @'Fun' a b@,
@@ -170,6 +171,8 @@ module Test.QuickCheck
   , functionIntegral
   , functionRealFrac
   , functionBoundedEnum
+  , functionVoid
+#endif
 
     -- * The 'CoArbitrary' typeclass: generation of functions the old-fashioned way
   , CoArbitrary(..)
@@ -220,8 +223,10 @@ module Test.QuickCheck
   , InfiniteList(..)
   , SortedList(..)
   , Positive(..)
+  , Negative(..)
   , NonZero(..)
   , NonNegative(..)
+  , NonPositive(..)
   , Large(..)
   , Small(..)
   , Smart(..)
@@ -303,7 +308,9 @@ import Test.QuickCheck.Modifiers
 import Test.QuickCheck.Property hiding ( Result(..) )
 import Test.QuickCheck.Test
 import Test.QuickCheck.Exception
+#ifndef NO_GADTS
 import Test.QuickCheck.Function
+#endif
 import Test.QuickCheck.Features
 import Test.QuickCheck.State
 #ifndef NO_TEMPLATE_HASKELL
