@@ -1,4 +1,7 @@
 -- | Type classes for random generation of values.
+--
+-- __Note__: the contents of this module are re-exported by
+-- "Test.QuickCheck". You do not need to import it directly.
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
 #ifndef NO_GENERICS
@@ -63,6 +66,7 @@ module Test.QuickCheck.Arbitrary
   , shrinkMapBy              -- :: (a -> b) -> (b -> a) -> (a -> [a]) -> b -> [b]
   , shrinkIntegral           -- :: Integral a => a -> [a]
   , shrinkRealFrac           -- :: RealFrac a => a -> [a]
+  , shrinkDecimal            -- :: RealFrac a => a -> [a]
   -- ** Helper functions for implementing coarbitrary
   , coarbitraryIntegral      -- :: Integral a => a -> Gen b -> Gen b
   , coarbitraryReal          -- :: Real a => a -> Gen b -> Gen b
@@ -489,7 +493,7 @@ instance (RealFloat a, Arbitrary a) => Arbitrary (Complex a) where
 #ifndef NO_FIXED
 instance HasResolution a => Arbitrary (Fixed a) where
   arbitrary = arbitrarySizedFractional
-  shrink    = shrinkRealFrac
+  shrink    = shrinkDecimal
 #endif
 
 instance Arbitrary2 (,) where
@@ -630,7 +634,7 @@ instance Arbitrary Int64 where
   shrink    = shrinkIntegral
 
 instance Arbitrary Word where
-  arbitrary = arbitrarySizedBoundedIntegral
+  arbitrary = arbitrarySizedIntegral
   shrink    = shrinkIntegral
 
 instance Arbitrary Word8 where
@@ -673,11 +677,11 @@ instance Arbitrary Char where
 
 instance Arbitrary Float where
   arbitrary = arbitrarySizedFractional
-  shrink    = shrinkRealFrac
+  shrink    = shrinkDecimal
 
 instance Arbitrary Double where
   arbitrary = arbitrarySizedFractional
-  shrink    = shrinkRealFrac
+  shrink    = shrinkDecimal
 
 instance Arbitrary CChar where
   arbitrary = arbitrarySizedBoundedIntegral
@@ -755,62 +759,35 @@ instance Arbitrary CUIntMax where
   arbitrary = arbitrarySizedBoundedIntegral
   shrink = shrinkIntegral
 
-#ifndef NO_NEWTYPE_DERIVING
+#ifndef NO_CTYPES_CONSTRUCTORS
 -- The following four types have no Bounded instance,
 -- so we fake it by discovering the bounds at runtime.
 instance Arbitrary CClock where
-  arbitrary = fmap unBounds arbitrary
-  shrink = shrinkMap unBounds Bounds
+  arbitrary = fmap CClock arbitrary
+  shrink (CClock x) = map CClock (shrink x)
 
 instance Arbitrary CTime where
-  arbitrary = fmap unBounds arbitrary
-  shrink = shrinkMap unBounds Bounds
+  arbitrary = fmap CTime arbitrary
+  shrink (CTime x) = map CTime (shrink x)
 
 #ifndef NO_FOREIGN_C_USECONDS
 instance Arbitrary CUSeconds where
-  arbitrary = fmap unBounds arbitrary
-  shrink = shrinkMap unBounds Bounds
+  arbitrary = fmap CUSeconds arbitrary
+  shrink (CUSeconds x) = map CUSeconds (shrink x)
 
 instance Arbitrary CSUSeconds where
-  arbitrary = fmap unBounds arbitrary
-  shrink = shrinkMap unBounds Bounds
+  arbitrary = fmap CSUSeconds arbitrary
+  shrink (CSUSeconds x) = map CSUSeconds (shrink x)
 #endif
-
-newtype Bounds a = Bounds { unBounds :: a }
-  deriving (Eq, Ord, Num, Enum, Real, Show)
-
-instance (Ord a, Num a) => Bounded (Bounds a) where
-  -- assume max has all 1s in binary expansion
-  maxBound = maximum (nubIterate (\x -> 2*x+1) 1)
-  -- assume min has a leading 1 and rest 0s in binary expansion (or is 0)
-  minBound = minimum (0:nubIterate (*2) 1)
-
-instance (Num a, Real a, Enum a) => Integral (Bounds a) where
-  toInteger = fromIntegral . fromEnum
-  x `quotRem` y =
-    let (z, w) = toInteger x `quotRem` toInteger y in
-    (fromInteger z, fromInteger w)
-
-instance (Ord a, Num a, Real a, Enum a) => Arbitrary (Bounds a) where
-  arbitrary = arbitrarySizedBoundedIntegral
-  shrink = shrinkIntegral
-
--- Like iterate, but stop when you reach an existing value.
-nubIterate :: Eq a => (a -> a) -> a -> [a]
-nubIterate f x = iter [] x
-  where
-    iter xs x
-      | x `elem` xs = []
-      | otherwise = x:iter (x:xs) (f x)
 #endif
 
 instance Arbitrary CFloat where
   arbitrary = arbitrarySizedFractional
-  shrink = shrinkRealFrac
+  shrink = shrinkDecimal
 
 instance Arbitrary CDouble where
   arbitrary = arbitrarySizedFractional
-  shrink = shrinkRealFrac
+  shrink = shrinkDecimal
 
 -- Arbitrary instances for container types
 instance (Ord a, Arbitrary a) => Arbitrary (Set.Set a) where
@@ -1016,8 +993,8 @@ arbitrarySizedFractional :: Fractional a => Gen a
 arbitrarySizedFractional =
   sized $ \n ->
     let n' = toInteger n in
-      do a <- choose ((-n') * precision, n' * precision)
-         b <- choose (1, precision)
+      do b <- choose (1, precision)
+         a <- choose ((-n') * b, n' * b)
          return (fromRational (a % b))
  where
   precision = 9999999999999 :: Integer
@@ -1058,8 +1035,15 @@ arbitrarySizedBoundedIntegral =
   sized $ \s ->
     do let bits n | n == 0 = 0
                   | otherwise = 1 + bits (n `quot` 2)
-           k  = 2^(s*(bits mn `max` bits mx `max` 40) `div` 80)
-       n <- choose (toInteger mn `max` (-k), toInteger mx `min` k)
+           k = (toInteger s*(bits mn `max` bits mx `max` 40) `div` 80)
+           -- computes x `min` (2^k), but avoids computing 2^k
+           -- if it is too large
+           x `minexp` k
+             | bits x < k = x
+             | otherwise = x `min` (2^k)
+           -- x `max` (-2^k)
+           x `maxexpneg` k = -((-x) `minexp` k)
+       n <- choose (toInteger mn `maxexpneg` k, toInteger mx `minexp` k)
        return (fromInteger n)
 
 -- ** Generators for various kinds of character
@@ -1120,20 +1104,54 @@ shrinkIntegral x =
             (True,  False) -> a + b < 0
             (False, True)  -> a + b > 0
 
--- | Shrink a fraction.
+-- | Shrink a fraction, preferring numbers with smaller
+-- numerators or denominators. See also 'shrinkDecimal'.
 shrinkRealFrac :: RealFrac a => a -> [a]
-shrinkRealFrac x =
-  nub $
-  [ -x
-  | x < 0
-  ] ++
-  map fromInteger (shrinkIntegral (truncate x))
+shrinkRealFrac x
+  | not (x == x)  = 0 : take 10 (iterate (*2) 0) -- NaN
+  | not (2*x+1>x) = 0 : takeWhile (<x) (iterate (*2) 0) -- infinity
+  | x < 0 = negate x:map negate (shrinkRealFrac (negate x))
+  | otherwise =
+    -- To ensure termination
+    filter (\y -> abs y < abs x) $
+      -- Try shrinking to an integer first
+      map fromInteger (shrink (truncate x) ++ [truncate x]) ++
+      -- Shrink the numerator
+      [fromRational (num' % denom) | num' <- shrink num] ++
+      -- Shrink the denominator, and keep the fraction as close
+      -- to the original as possible, rounding towards zero
+      [fromRational (truncate (num * denom' % denom) % denom')
+      | denom' <- shrink denom, denom' /= 0 ]
+  where
+    num = numerator (toRational x)
+    denom = denominator (toRational x)
+
+-- | Shrink a real number, preferring numbers with shorter
+-- decimal representations. See also 'shrinkRealFrac'.
+shrinkDecimal :: RealFrac a => a -> [a]
+shrinkDecimal x
+  | not (x == x)  = 0 : take 10 (iterate (*2) 0)        -- NaN
+  | not (2*x+1>x) = 0 : takeWhile (<x) (iterate (*2) 0) -- infinity
+  | otherwise =
+    -- e.g. shrink pi =
+    --   shrink 3 ++ map (/ 10) (shrink 31) ++
+    --   map (/ 100) (shrink 314) + ...,
+    -- where the inner calls to shrink use integer shrinking.
+    [ y
+    | precision <- take 6 (iterate (*10) 1),
+      let m = round (toRational x * precision),
+      m `mod` 10 /= 0, -- don't allow shrinking to increase digits
+      n <- m:shrink m,
+      let y = fromRational (fromInteger n / precision),
+      abs y < abs x ]
 
 --------------------------------------------------------------------------
 -- ** CoArbitrary
 
 #ifndef NO_GENERICS
 -- | Used for random generation of functions.
+-- You should consider using 'Test.QuickCheck.Fun' instead, which
+-- can show the generated functions as strings.
 --
 -- If you are using a recent GHC, there is a default definition of
 -- 'coarbitrary' using 'genericCoarbitrary', so if your type has a
