@@ -94,6 +94,16 @@ class Testable prop where
   -- | Convert the thing to a property.
   property :: prop -> Property
 
+  -- | Optional; used internally in order to improve shrinking.
+  -- Tests a property but also quantifies over an extra value
+  -- (with a custom shrink and show function).
+  -- The 'Testable' instance for functions defines
+  -- @propertyForAllShrinkShow@ in a way that improves shrinking.
+  propertyForAllShrinkShow :: Gen a -> (a -> [a]) -> (a -> [String]) -> (a -> prop) -> Property
+  propertyForAllShrinkShow gen shr shw f =
+    forAllShrinkBlind gen shr $
+      \x -> foldr counterexample (property (f x)) (shw x)
+
 -- | If a property returns 'Discard', the current test case is discarded,
 -- the same as if a precondition was false.
 --
@@ -115,6 +125,13 @@ instance Testable () where
       -- so that we turn exceptions into test failures
       liftUnit () = succeeded
 
+instance Testable prop => Testable (Maybe prop) where
+  property = property . liftMaybe
+    where
+      -- See comment for liftUnit above
+      liftMaybe Nothing = property Discard
+      liftMaybe (Just prop) = property prop
+
 instance Testable Bool where
   property = property . liftBool
 
@@ -122,13 +139,13 @@ instance Testable Result where
   property = MkProperty . return . MkProp . protectResults . return
 
 instance Testable Prop where
-  property (MkProp r) = MkProperty . return . MkProp . ioRose . return $ r
+  property p = MkProperty . return . protectProp $ p
 
 instance Testable prop => Testable (Gen prop) where
   property mp = MkProperty $ do p <- mp; unProperty (again p)
 
 instance Testable Property where
-  property (MkProperty mp) = MkProperty $ do p <- mp; unProperty (property p)
+  property (MkProperty mp) = MkProperty (fmap protectProp mp)
 
 -- | Do I/O inside a property.
 {-# DEPRECATED morallyDubiousIOProperty "Use 'ioProperty' instead" #-}
@@ -157,7 +174,16 @@ idempotentIOProperty =
   promote . fmap (unProperty . property)
 
 instance (Arbitrary a, Show a, Testable prop) => Testable (a -> prop) where
-  property f = forAllShrink arbitrary shrink f
+  property f =
+    propertyForAllShrinkShow arbitrary shrink (return . show) f
+  propertyForAllShrinkShow gen shr shw f =
+    -- gen :: Gen b, shr :: b -> [b], f :: b -> a -> prop
+    -- Idea: Generate and shrink (b, a) as a pair
+    propertyForAllShrinkShow
+      (liftM2 (,) gen arbitrary)
+      (liftShrink2 shr shrink)
+      (\(x, y) -> shw x ++ [show y])
+      (uncurry f)
 
 -- ** Exception handling
 protect :: (AnException -> a) -> IO a -> IO a
@@ -216,6 +242,10 @@ onRose f (IORose m) = IORose (fmap (onRose f) m)
 protectRose :: IO (Rose Result) -> IO (Rose Result)
 protectRose = protect (return . exception "Exception")
 
+-- | Wrap the top level of a 'Prop' in an exception handler.
+protectProp :: Prop -> Prop
+protectProp (MkProp r) = MkProp (IORose . protectRose . return $ r)
+
 -- | Wrap all the Results in a rose tree in exception handlers.
 protectResults :: Rose Result -> Rose Result
 protectResults = onRose $ \x rs ->
@@ -305,7 +335,7 @@ succeeded, failed, rejected :: Result
 
 liftBool :: Bool -> Result
 liftBool True = succeeded
-liftBool False = failed { reason = "Falsifiable" }
+liftBool False = failed { reason = "Falsified" }
 
 mapResult :: Testable prop => (Result -> Result) -> prop -> Property
 mapResult f = mapRoseResult (protectResults . fmap f)
@@ -505,7 +535,9 @@ stdConfidence =
 -- not what you want, use 'tabulate'.
 label :: Testable prop => String -> prop -> Property
 label s =
+#ifndef NO_DEEPSEQ
   s `deepseq`
+#endif
   mapTotalResult $
     \res -> res { labels = s:labels res }
 
@@ -553,7 +585,9 @@ classify :: Testable prop =>
          -> prop -> Property
 classify False _ = property
 classify True s =
+#ifndef NO_DEEPSEQ
   s `deepseq`
+#endif
   mapTotalResult $
     \res -> res { classes = s:classes res }
 
@@ -654,7 +688,9 @@ cover p x s = mapTotalResult f . classify x s
 -- 16% LogOut
 tabulate :: Testable prop => String -> [String] -> prop -> Property
 tabulate key values =
+#ifndef NO_DEEPSEQ
   key `deepseq` values `deepseq`
+#endif
   mapTotalResult $
     \res -> res { tables = [(key, value) | value <- values] ++ tables res }
 
@@ -704,7 +740,9 @@ tabulate key values =
 coverTable :: Testable prop =>
   String -> [(String, Double)] -> prop -> Property
 coverTable table xs =
-  tables `deepseq` xs `deepseq`
+#ifndef NO_DEEPSEQ
+  table `deepseq` xs `deepseq`
+#endif
   mapTotalResult $
     \res -> res { requiredCoverage = ys ++ requiredCoverage res }
   where
@@ -721,6 +759,16 @@ True  ==> p = property p
 
 -- | Considers a property failed if it does not complete within
 -- the given number of microseconds.
+--
+-- Note: if the property times out, variables quantified inside the
+-- `within` will not be printed. Therefore, you should use `within`
+-- only in the body of your property.
+--
+-- Good: @prop_foo a b c = within 1000000 ...@
+--
+-- Bad: @prop_foo = within 1000000 $ \\a b c -> ...@
+--
+-- Bad: @prop_foo a b c = ...; main = quickCheck (within 1000000 prop_foo)@
 within :: Testable prop => Int -> prop -> Property
 within n = mapRoseResult f
   where
