@@ -1,6 +1,9 @@
 {-# OPTIONS_HADDOCK hide #-}
 -- | The main test loop.
 {-# LANGUAGE CPP #-}
+#ifndef NO_TYPEABLE
+{-# LANGUAGE DeriveDataTypeable #-}
+#endif
 #ifndef NO_SAFE_HASKELL
 {-# LANGUAGE Trustworthy #-}
 #endif
@@ -17,7 +20,6 @@ import Test.QuickCheck.State hiding (labels, classes, tables, requiredCoverage)
 import qualified Test.QuickCheck.State as S
 import Test.QuickCheck.Exception
 import Test.QuickCheck.Random
-import Data.Number.Erf(invnormcdf)
 import System.Random(split)
 #if defined(MIN_VERSION_containers)
 #if MIN_VERSION_containers(0,5,0)
@@ -41,16 +43,18 @@ import Data.List
   , sortBy
   , group
   , intersperse
-  , intercalate
   )
 
 import Data.Maybe(fromMaybe, isNothing, catMaybes)
 import Data.Ord(comparing)
 import Text.Printf(printf)
-import Data.Either(lefts, rights)
 import Control.Monad
 import Data.Bits
 
+#ifndef NO_TYPEABLE
+import Data.Typeable (Typeable)
+#endif
+
 --------------------------------------------------------------------------
 -- quickCheck
 
@@ -79,7 +83,11 @@ data Args
     -- ^ Maximum number of shrinks to before giving up. Setting this to zero
     --   turns shrinking off.
   }
- deriving ( Show, Read )
+ deriving ( Show, Read
+#ifndef NO_TYPEABLE
+  , Typeable
+#endif
+  )
 
 -- | Result represents the test result
 data Result
@@ -167,6 +175,12 @@ stdArgs = Args
 --
 -- By default up to 100 tests are performed, which may not be enough
 -- to find all bugs. To run more tests, use 'withMaxSuccess'.
+--
+-- If you want to get the counterexample as a Haskell value,
+-- rather than just printing it, try the
+-- <http://hackage.haskell.org/package/quickcheck-with-counterexamples quickcheck-with-counterexamples>
+-- package.
+
 quickCheck :: Testable prop => prop -> IO ()
 quickCheck p = quickCheckWith stdArgs p
 
@@ -390,16 +404,16 @@ failureSummaryAndReason st res = (summary, full)
   where
     summary =
       header ++
-      short 26 (oneLine reason ++ " ") ++
+      short 26 (oneLine theReason ++ " ") ++
       count True ++ "..."
 
     full =
       (header ++
-       (if isOneLine reason then reason ++ " " else "") ++
+       (if isOneLine theReason then theReason ++ " " else "") ++
        count False ++ ":"):
-      if isOneLine reason then [] else lines reason
+      if isOneLine theReason then [] else lines theReason
 
-    reason = P.reason res
+    theReason = P.reason res
 
     header =
       if expect res then
@@ -433,10 +447,10 @@ success st = do
           (":":short, long)
 
 labelsAndTables :: State -> ([String], [String])
-labelsAndTables st = (labels, tables)
+labelsAndTables st = (theLabels, theTables)
   where
-    labels :: [String]
-    labels =
+    theLabels :: [String]
+    theLabels =
       paragraphs $
         [ showTable (numSuccessTests st) Nothing m
         | m <- S.classes st:Map.elems numberedLabels ]
@@ -448,8 +462,8 @@ labelsAndTables st = (labels, tables)
         | (labels, n) <- Map.toList (S.labels st),
           (i, l) <- zip [0..] labels ]
 
-    tables :: [String]
-    tables =
+    theTables :: [String]
+    theTables =
       paragraphs $
         [ showTable (sum (Map.elems m)) (Just table) m
         | (table, m) <- Map.toList (S.tables st) ] ++
@@ -573,6 +587,67 @@ wilsonLow k n a = wilson k n (invnormcdf (a/2))
 wilsonHigh :: Integer -> Integer -> Double -> Double
 wilsonHigh k n a = wilson k n (invnormcdf (1-a/2))
 
+-- Algorithm taken from
+-- https://web.archive.org/web/20151110174102/http://home.online.no/~pjacklam/notes/invnorm/
+-- Accurate to about one part in 10^9.
+--
+-- The 'erf' package uses the same algorithm, but with an extra step
+-- to get a fully accurate result, which we skip because it requires
+-- the 'erfc' function.
+invnormcdf :: Double -> Double
+invnormcdf p
+  | p < 0  = 0/0
+  | p > 1  = 0/0
+  | p == 0 = -1/0
+  | p == 1 = 1/0
+  | p < p_low =
+    let
+      q = sqrt(-2*log(p))
+    in
+      (((((c1*q+c2)*q+c3)*q+c4)*q+c5)*q+c6) /
+      ((((d1*q+d2)*q+d3)*q+d4)*q+1)
+  | p <= p_high =
+    let
+      q = p - 0.5
+      r = q*q
+    in
+      (((((a1*r+a2)*r+a3)*r+a4)*r+a5)*r+a6)*q /
+      (((((b1*r+b2)*r+b3)*r+b4)*r+b5)*r+1)
+  | otherwise =
+    let
+      q = sqrt(-2*log(1-p))
+    in
+      -(((((c1*q+c2)*q+c3)*q+c4)*q+c5)*q+c6) /
+       ((((d1*q+d2)*q+d3)*q+d4)*q+1)
+  where
+    a1 = -3.969683028665376e+01
+    a2 =  2.209460984245205e+02
+    a3 = -2.759285104469687e+02
+    a4 =  1.383577518672690e+02
+    a5 = -3.066479806614716e+01
+    a6 =  2.506628277459239e+00
+
+    b1 = -5.447609879822406e+01
+    b2 =  1.615858368580409e+02
+    b3 = -1.556989798598866e+02
+    b4 =  6.680131188771972e+01
+    b5 = -1.328068155288572e+01
+
+    c1 = -7.784894002430293e-03
+    c2 = -3.223964580411365e-01
+    c3 = -2.400758277161838e+00
+    c4 = -2.549732539343734e+00
+    c5 =  4.374664141464968e+00
+    c6 =  2.938163982698783e+00
+
+    d1 =  7.784695709041462e-03
+    d2 =  3.224671290700398e-01
+    d3 =  2.445134137142996e+00
+    d4 =  3.754408661907416e+00
+
+    p_low  = 0.02425
+    p_high = 1 - p_low
+
 addCoverageCheck :: Confidence -> State -> Property -> Property
 addCoverageCheck confidence st prop
   | and [ sufficientlyCovered confidence tot n p
@@ -581,9 +656,9 @@ addCoverageCheck confidence st prop
     once prop
   | or [ insufficientlyCovered (Just (certainty confidence)) tot n p
        | (_, _, tot, n, p) <- allCoverage st ] =
-    let (labels, tables) = labelsAndTables st in
+    let (theLabels, theTables) = labelsAndTables st in
     foldr counterexample (property failed{P.reason = "Insufficient coverage"})
-      (paragraphs [labels, tables])
+      (paragraphs [theLabels, theTables])
   | otherwise = prop
 
 allCoverage :: State -> [(Maybe String, String, Int, Int, Double)]
